/* $Workfile: Dsp64Api.h $   $Revision: 3 $    $Modtime: 1/20/09 4:27a $  */
#ifndef DSP64API_OTX__DEFINED
#define DSP64API_OTX__DEFINED

/*******************************************************************************

  @doc PublicDsp55SdkApi 

  Copyright (C) 2008 - 2009 Odin TeleSystems Inc., All Rights Reserved

  SAA-10xx-1 OTX DSP C64x Software Development Kit (SDK)

  DSP SDK API Functions

  @module
  Dsp64Api | DSK SDK API Function Prototypes

  @contents2
  Dsp64Api.h |

  @index func | Dsp64Api

  @contents2
  DSP SDK Functions (Dsp64Api.h) | DSP SDK Functions available for custom DSP Applications

  @head3 Dsp64Api Function Reference |

  @index func | Dsp64Api

*******************************************************************************/


#include "Dsp64XdmLink.h"

#include "Dsp64Types.h"

#ifdef __cplusplus
  extern "C" {
#endif
#include <ti/sdo/ce/visa.h>

struct OtxDsp64CallContextS {
  unsigned short *pEventBuf;       // Ptr to events buf 
  unsigned long coEventBuf;        // Size of events buffer (IN)
  unsigned long coBytesRet;        // Current position in the buffer
  unsigned long coEvents;          // Number of events already in the buffer

  unsigned short *pIoInBuf;        // Output buffer (Io control requests)
  unsigned short *pIoOutBuf;       // Output buffer (Io control requests)
  unsigned short coIoCtlOutBuf;    // Out Ioctl buffer size
  unsigned short coIoCtlRet;       // Out Ioctl buffer size

  Engine_Handle  hEngine;          // ptr to a VISA_Obj* = VISA_Handle. Need to turn to the (VISA_Handle) in the user's code
                                   // To be used for create an XDM algorithms inside of the XDMLINK Dsp codec and user's callback.

  XDAS_Int32     nSeqNo;           // Sequence number. For multiIO control calls         
            
     // Reserved fields ... Note this structure should be aligned on the 16 bits (sanity check).
  
  XDAS_Int32 *pIoHeader;  
}; 

typedef struct OtxDsp64CallContextS OtxDsp64CallContextS;


// Global Call Context

extern OtxDsp64CallContextS *g_pDsp64CallContext;

/*--------------------------------------------------------------------------*\
  API FUNCTION DEFINITIONS
\*--------------------------------------------------------------------------*/
/*
 // Turn on D1/D3 (marked A) LED
  REG(CLR_DATA01_REG) = ALVIS_LED_D1;

  // Turn off D1/D3 (marked A) LED
  REG(SET_DATA01_REG) = ALVIS_LED_D1;
#define REG(addr) (*(volatile unsigned int *)(addr))

#define SET_DATA01_REG (0x01C67018) 
#define CLR_DATA01_REG (0x01C6701C)

*/

/*******************************************************************************
  @func
    Run Standard OTX DSP SDK API Initializations. Calls 
    <f OtxDspInitSerialPort>.
*******************************************************************************/
void OtxDsp64Init(
  void                          /* @parm No parameters */
);


/*******************************************************************************
  @func
    Initialize Standard Global Data. Called by <f OtxDspInit>.
*******************************************************************************/
void OtxDsp64InitGlobalData(
  void                          /* @parm No parameters */
);


/*******************************************************************************
  @func
    Verifies that the DSP SDK and the Host driver revisions match. Initializes
    the PLL to make it run at the maximum speed for this particular DSP.

    This function should be called when the OTXDSP_SDK_IO_START_PROGRAM IO code
    is sent from the host. 
    
  @xref
    <t OtxDspSdkIoInStartProgramS>

*******************************************************************************/
#if 0
short OtxDsp64StartProgram(
  OtxDspSdkIoInStartProgramS *pSin  /* @parm  */
);

#endif

#if 1
  #define OtxDspCommandCompleteData(a,b,c) OtxDsp64CommandCompleteData(g_pDsp64CallContext,a,b,c)
  #define OtxDspCommandCompleteRc(a,b) OtxDsp64CommandCompleteRc(g_pDsp64CallContext,a,b)
  #define OtxDspCommandComplete(a) OtxDsp64CommandComplete(g_pDsp64CallContext,a)


  #define OtxDspSetHostEventVoid(a1,a2) OtxDsp64SetHostEventVoid(g_pDsp64CallContext, a1, a2);
  #define OtxDspSetHostEvent(a1,a2,a3,a4,a5) OtxDsp64SetHostEvent(g_pDsp64CallContext, a1, a2, a3, a4, a5)
  #define OtxDspSetHostEventEx(a1,a2,a3,a4,a5,a6) OtxDsp64SetHostEventEx(g_pDsp64CallContext, a1, a2, a3, a4, a5, a6)
  #define OtxDspSetHostEventUlong(a1,a2,a3) OtxDsp64SetHostEventUlong(g_pDsp64CallContext, a1, a2, a3)
  #define OtxDspSetHostEventUshort(a1,a2,a3) OtxDsp64SetHostEventUshort(g_pDsp64CallContext, a1, a2, a3)
#else
//#ifdef OTX_DSP64SDK_API_SUBSTITUTE_NAMES
  #define OtxDspCommandCompleteData(a,b,c) OtxDsp64CommandCompleteData(pContext,a,b,c)
  #define OtxDspCommandCompleteRc(a,b) OtxDsp64CommandCompleteRc(pContext,a,b)
  #define OtxDspCommandComplete(a) OtxDsp64CommandComplete(pContext,a)

  #define OtxDspSetHostEvent(a1,a2,a3,a4,a5) OtxDsp64SetHostEvent(pContext, a1, a2, a3, a4, a5)
  #define OtxDspSetHostEventEx(a1,a2,a3,a4,a5,a6) OtxDsp64SetHostEventEx(pContext, a1, a2, a3, a4, a5, a6)
  #define OtxDspSetHostEventUlong(a1,a2,a3) OtxDsp64SetHostEventUlong(pContext, a1, a2, a3)
  #define OtxDspSetHostEventUshort(a1,a2,a3) OtxDsp64SetHostEventUshort(pContext, a1, a2, a3)
#endif

  
/*******************************************************************************
  @func
    Completes an IO Control call from the host. Passes an
    arbitrary size of return data from a IO Control call back to the host.
*******************************************************************************/
void OtxDsp64CommandCompleteDataBytes(
  OtxDsp64CallContextS *pContext, 
  DSP_OTX_HANDLE hHandle,       /* @parm Handle to the user logical device */
  unsigned short *pData,        /* @parm Pointer to the data to be returned to the host driver */
  unsigned short dataLen        /* @parm Length of the data to be returned to the host driver - in Bytes!!! */
); 


/*******************************************************************************
  @func
    Completes an IO Control call from the host. Passes an
    arbitrary size of return data from a IO Control call back to the host.
*******************************************************************************/
inline void OtxDsp64CommandCompleteDataWords(
  OtxDsp64CallContextS *pContext, 
  DSP_OTX_HANDLE hHandle,       /* @parm Handle to the user logical device */
  unsigned short *pData,        /* @parm Pointer to the data to be returned to the host driver */
  unsigned short cbData        /* @parm Length of the data to be returned to the host driver - in Words!!! */
)
{
  OtxDsp64CommandCompleteDataBytes(pContext,hHandle, pData, cbData*2);
}


/*******************************************************************************
  @func
    Completes an IO Control call from the host. Passes an
    arbitrary size of return data from a IO Control call back to the host.
*******************************************************************************/
inline void OtxDsp64CommandCompleteData(
  OtxDsp64CallContextS *pContext, 
  DSP_OTX_HANDLE hHandle,       /* @parm Handle to the user logical device */
  unsigned short *pData,        /* @parm Pointer to the data to be returned to the host driver */
  unsigned short cbData        /* @parm Length of the data to be returned to the host driver - in Words!!! */
)
{
  OtxDsp64CommandCompleteDataBytes(pContext,hHandle, pData, cbData);
}


/*******************************************************************************
  @func
    Completes an IO Control call from the host. Passes a 
    return code from an IO Control call back to the host.
*******************************************************************************/
inline void OtxDsp64CommandCompleteRc(
  OtxDsp64CallContextS *pContext, 
  DSP_OTX_HANDLE hHandle,       /* @parm Handle to the user defined logical device */
  unsigned short nResult        /* @parm Result code to be passed to the host driver */
){
  OtxDsp64CommandCompleteDataWords(pContext, hHandle, &nResult,1); // datalen in words!!
}


/*******************************************************************************
  @func
    Completes an IO Control call from the host. No data is passed back
    to the host.
*******************************************************************************/
inline void OtxDsp64CommandComplete(
  OtxDsp64CallContextS *pContext, 
  DSP_OTX_HANDLE       hHandle        /* @parm Handle to the user defined logical device */
){
  OtxDsp64CommandCompleteData(pContext, hHandle, NULL,0);
}


/*******************************************************************************
  @func
    Takes in a 14 bit 2-s complement value and returns a A-law compressed
    value ready to transmission (8-bits)
  @comm
    Every other bit is inverted (for transmission).
*******************************************************************************/
extern short OtxDspAlawCompress(
  short lawCompInput            /* @parm 14 bit 2-s complement value to be companded */
);


/*******************************************************************************
  @func
    Takes in a 14 bit 2-s complement value and returns a U-law compressed
    value ready to transmission (8-bits)
  @comm
    Value bits (0-6) are inverted
*******************************************************************************/
extern short OtxDspUlawCompress(
  short lawCompInput            /* @parm 14 bit 2-s complement value to be companded */
);


/*******************************************************************************
  @func
    Expands a U-law or A-law companded 8-bit sample and returns a 14-bit 2's
    complement value (in the 14 least significant bits of a 16-bit word)
  @comm
    Since a-law only has a range of -4196 .. x .. 4195, the value is multiplied
    by 2 before it is returned to make it a full scale 14-bit value.
*******************************************************************************/
extern short OtxDspLawExpand(
  unsigned short lawExpandInput,/* @parm Sample to be expanded (8-bits sign magnitude value */
  short law                     /* @parm Law (0 = u-law, 1 = A-law) */
);


/*******************************************************************************
  @func
    Calculates the amount of free space in the notify event buffer. 
    The result ends up in the 'g_hpiEventBufDiag' variable.
*******************************************************************************/
void OtxDspCalculateFreeBufferSpace(
  void
);


/*--------------------------------------------------------------------------*\
  API CALL-BACK FUNCTIONS
\*--------------------------------------------------------------------------*/

/*******************************************************************************
  @func
    Generates a driver event for the host and copies the associated data
    to the HPI buffer.
*******************************************************************************/
short OtxDsp64SetHostEvent(
  OtxDsp64CallContextS *pContext, 
  unsigned short nEvent,        /* @parm Event to be send to the host */
  DSP_OTX_HANDLE hHandle,       /* @parm Handle to the user logical device sending the event */
  unsigned short nTaskRef,      /* @parm Associated Task Reference */
  unsigned short *pData,        /* @parm Pointer to the data to be sent to the host */
  unsigned short nDataLen       /* @parm Length of the data to be sent */
);


/*******************************************************************************
  @func
    Generates a driver event for the host and copies the associated data
    to the HPI buffer.

    Extended version. Has the ability to transfer a buffer of octets as
    well as a buffer of words.

*******************************************************************************/
short OtxDsp64SetHostEventEx(
  OtxDsp64CallContextS *pContext, 
  unsigned short nEvent,        /* @parm Event to be send to the host */
  DSP_OTX_HANDLE hHandle,       /* @parm Handle to the user logical device sending the event */
  unsigned short nTaskRef,      /* @parm Associated Task Reference */
  unsigned short *pData,        /* @parm Pointer to the data to be sent to the host */
  unsigned short nDataLen,      /* @parm Length of the data to be sent */
  short bOctetTransfer          /* @parm OTXDSP_TRUE if nDataLen and pData specifies octets. OTXDSP_FALSE if they specify words */
);


/*******************************************************************************
  @func
    Easy access function for sending an event to the host with unsigned
    long as the data.
*******************************************************************************/
short OtxDsp64SetHostEventUlong(
  OtxDsp64CallContextS *pContext, 
  unsigned short nEvent,        /* @parm Event to be send to the host */
  DSP_OTX_HANDLE hHandle,       /* @parm Handle to the user logical device sending the event */
  unsigned long data            /* @parm Data to be sent to the host */
);


/*******************************************************************************
  @func
    Easy access function for sending an event to the host with an unsigned
    short as the data.
*******************************************************************************/
short OtxDsp64SetHostEventUshort(
  OtxDsp64CallContextS *pContext, 
  unsigned short nEvent,        /* @parm Event to be send to the host */                       
  DSP_OTX_HANDLE hHandle,       /* @parm Handle to the user logical device sending the event */
  unsigned short data           /* @parm Data to be sent to the host */                        
);


/*******************************************************************************
  @func
    Easy access function for sending an event to the host with no data.
*******************************************************************************/
short OtxDsp64SetHostEventVoid(
  OtxDsp64CallContextS *pContext, 
  unsigned short nEvent,        /* @parm Event to be send to the host */                       
  DSP_OTX_HANDLE hHandle        /* @parm Handle to the user logical device sending the event */
);                               


/*******************************************************************************
  @func

  @comm
    Debug printf
*******************************************************************************/

#ifdef DEBUG_PRINTFS
short DPrintf(char *fmt, ...);
#define dprintf DPrintf
#endif

#if 0

void Dprintf(char *fmt, ...);
#define dprintf Dprintf

#else

//  void DPrintf(char *fmt, ...);
  void DPrintf1(char *str);
  extern char *Dprintf_buf;
//#define Dprintf(s,...) sprintf(Dprintf_buf,s,...); Dprintf1(Dprintf_buf)
//#define Dprintf dprintf

#define DPrintf_0(str)                               DPrintf(str)
#define DPrintf_1(fmt,val0)                          DPrintf(fmt,val0)
#define DPrintf_2(fmt,val0,val1)                     DPrintf(fmt,val0,val1)
#define DPrintf_3(fmt,val0,val1,val2)                DPrintf(fmt,val0,val1,val2)
#define DPrintf_4(fmt,val0,val1,val2,val3)           DPrintf(fmt,val0,val1,val2,val3)
#define DPrintf_5(fmt,val0,val1,val2,val3,val4)      DPrintf(fmt,val0,val1,val2,val3,val4)
#define DPrintf_6(fmt,val0,val1,val2,val3,val4,val5) DPrintf(fmt,val0,val1,val2,val3,val4,val5)

#endif
//#define printf dprintf

/*******************************************************************************
  @func

  @comm
    This function is called from the Interrupt Service Routine.
    TO BE RE-IMPLEMENTED BY THE USER.
*******************************************************************************/
void OtxDsp64ServiceIoControl(
  OtxDsp64CallContextS *pContext 
);


/*******************************************************************************
  @func

  @comm
    This function is called from the Interrupt Service Routine
    TO BE RE-IMPLEMENTED BY THE USER.
*******************************************************************************/
void OtxDsp64ServiceSerialPort(
  OtxDsp64CallContextS *pContext, 
  unsigned short *pRxBuf, unsigned short *pTxBuf, unsigned short nBufSize 
);




#ifdef __cplusplus
  }
#endif


/******************************************************************************

  Copyright (C) Odin TeleSystems Inc., 2008 - 2009


  All rights reserved. No part of this computer program may be reproduced,
  stored in any retrieval system, or transmitted, in any form or by any
  means, electronic, mechanical, photocopying, recording, or otherwise,
  without the prior written permission of Odin TeleSystems Inc., or without
  a permission stipulated in the agreement/contract under which this computer
  program was supplied.

******************************************************************************/


#endif


