/* $Workfile: OdSpmI.h $   $Revision: 1 $    $Modtime: 7/02/09 7:46a $  */
#ifndef DSP_OTXSPM_IO_H
#define DSP_OTXSPM_IO_H

/*------------------------------------------------------- File Description -*\

  Copyright (C) Odin TeleSystems Inc., 1999-2009

  DESCRIPTION

    Macros and structs that are shared between the DSP and the host for 
    all SPM modules.

    Please use caution when using struct because of differences in
    the way the DSP and the host these handle data types.

    The DSP says that sizeof(short) = 1 The host says that sizeof(short) =
    2 for 16-bit apps and 2 for 32-bit apps. We take care of this
    problem by dividing the sizeof(theStruct) with sizof(short). This is
    OK as long as we only have short in the struct.

    Note also that the DSP treats a pointer as a 16
    bit value. The host will treat a pointer as a 32 bit
    value. 

\*--------------------------------------------------------------------------*/

#if defined(_TMS320C5XX)
// No pack pragma needed for the DSP compiler
#elif defined(__TMS320C55X__)
// No pack pragma needed for the DSP compiler
#elif defined(xdc_target_name__) && xdc_target_name__==C64P
// No pack pragma needed for the DSP compiler
#elif defined (__linux__)
// GNU C has no pragma pack push support
#pragma pack(2)
#else
#pragma pack(push,2)
#endif


// The hDspDevHandle for Otx SPMs is a combination of DeviceType and DeviceNo
// The DeviceType is the 8 most significant bits of hDspDevHandle
// The DeviceNo is the 8 least significant bits of hDspDevHandle, which will uniquely identify the logical device (within one DSP)


#define SPM1_NO_HANDLE   0xFFFF
#define SPM1_NO_TASKREF  0xFFFF

#define SPM1_NO_LDEVICE_TYPE  0xFF00
#define SPM1_NO_LDEVICE_NUM   0x00FF

#define SPM1_MASK_LDEVICE_TYPE  0xFF00
#define SPM1_MASK_LDEVICE_NUM   0x00FF

#define SPM1_MASK_LDEVICE_TYPE_EX  0xFFFF0000L
#define SPM1_MASK_LDEVICE_NUM_EX   0x0000FFFFL



#define CH_TX_DIGITS_SIZE     34   // Used by multipe modules. Can be any size <= HOST2DSP_DATA_BUF_SIZE

#ifndef CH_DATA_SIZE
  #define CH_DATA_SIZE          102  // Used by Spm1, Spm2, Di
#endif

// The host will now query the SPM module for the ping/pong buffer size
//#define HDLC_SENDER_PING_PONG_BUF_SIZE  256



// We store DTMF digits and FSK octets in the same buffer. The following codes 
// are used (the upper byte of the buffer word) to distinguish between the 
// different types of data 
#define DTMF_DIGIT_CODE        1
#define FSK_OCTET_CODE         2

// Companding Law Type 
#define CLT_ULAW  0
#define CLT_ALAW  1

// Data store type for recording and playback 
//#define DATA_RAW      1   // Record/playback data as it appears on the transmission line
//#define DATA_COMPAND  2   // Record/playback data companded according to current companding law


// Data format types for data travelled on the highways 
// @todo Temporary, this must be the same values as OtxPipeDataFormatE
#define DF_UNKNOWN             0
#define DF_UINT8               1
#define DF_UINT16              2
#define DF_COMPAND8_A_LAW      3
#define DF_COMPAND8_U_LAW      4


// Which detector is running 
#define WD_DTMF 0
#define WD_FSK  1

// Tone effects
#define  TE_SINEWAVE       0  // @emem 1kHz sinusoidal wave
#define  TE_SAWTOOTH       1  // @emem Ramp wave
#define  TE_SWEEPING_SINE  2  // @emem Variable frequency sinusoidal wave
#define  TE_NOISE          3  // @emem Random noise
#define  TE_BYTE_VARIABLE  4  // @emem Sequential bytes: 0, 1, 2, ..., 255, 0, 1, ...

// Filter masks in the SS7 mode
#define SPM1_HDLC_SS7_FILTER_FISU           0x01
#define SPM1_HDLC_SS7_FILTER_SHORT_LSSU     0x02
#define SPM1_HDLC_SS7_FILTER_LONG_LSSU      0x04
#define SPM1_HDLC_SS7_FILTER_MSU            0x08
#define SPM1_HDLC_SS7_FILTER_FISU_ALL       0x10
#define SPM1_HDLC_SS7_FILTER_SHORT_LSSU_ALL 0x20
#define SPM1_HDLC_SS7_FILTER_LONG_LSSU_ALL  0x40

// Maximum length of the repeat frame (includes the CRC octets)
#define SPM1_HDLC_SENDER_REPEAT_BUF_SIZE    12  // Should match OTX_HDLC_MAX_REPEAT_DATA_LEN + CRC octets


#define SPM1_HB_ENDOFFRAME   0x01
#define SPM1_HB_ABORT        0x02
#define SPM1_HB_TRUNCATED    0x04

#define SPM1_SINEWAVE_ELEMENTS_MAX 5

#define SPM1_DTMF_DECODER_MATRIX_SIZE    5   // Note: There are lots of hardcoded values in the assembler code. Be very careful if we need to change this value 

#define SPM1_CO_MF_DECODER_FREQUENCIES  6
#define SPM1_CO_MF_ENCODER_FREQUENCIES  SPM1_CO_MF_DECODER_FREQUENCIES  

#define SPM1_CO_DUAL_DECODER_FREQUENCIES 6
#define SPM1_CO_SINGLE_DECODER_FREQUENCIES 6

#define SPM1_MF_ENCODER_MAX_DIGITS 34

#define SPM1_POTS_PULSE_GROUPS_MAX  128


enum Spm1CidErrorE
{
  // Generic errors.
  ODCID_NOERROR       = 0x00,  // No error.
  ODCID_TIMEOUT       = 0x01,  // Time out occures.
  ODCID_INVALID_STATE = 0x02,  // Invalid state.

  // Errors for CIDCHIP_FSK state.
  ODCID_CHECKSUMERROR = 0x10,  // Message checksum error.
  ODCID_ILLEGALLENGTH = 0x11,  // Illegal message length.
  ODCID_UNKNOWNTYPE   = 0x12,  // Unknown message type.

  // Otx additions
  ODCID_INVALIDNUMBER = 0x77   // Decoded number is invalid 
};

/*******************************************************************************
   @enum
     State of an SPM Device.
*******************************************************************************/
enum SpmDeviceStateE {
  SPM_STATE_UNKNOWN         = 0,     // @emem Device state is unknown, device has not been created
  SPM_STATE_CREATED         = 1,     // @emem Device has been created, but not yet connected
  SPM_STATE_CONNECTED       = 2,     // @emem Device is connected to the one or more timeslots, but not yet initialized
  SPM_STATE_INITIALIZED     = 3,     // @emem Device has been initialized (and created and connected), but not yet enabled
  SPM_STATE_WORKING_PENDING = 4,     // @emem Device will be operational when the enabling trigger point is reached
  SPM_STATE_WORKING         = 5,     // @emem Device is operational
  SPM_STATE_BLOCKED_PENDING = 6,     // @emem Device will be disabled when the disabling trigger point is reached
  SPM_STATE_BLOCKED_AUTO    = 7,     // @emem Device has been operational but is now automatically blocked due to an error condition
  SPM_STATE_BLOCKED_MANUAL  = 8      // @emem Device has been operational but is now manually blocked
};



// The logical device type is the 8 most significant bits of hDspDevHandle
// Don't use 0x0000 since that is defined as OTXDSP_SDK_LDEVICE_USER_APPLICATION   
#define SPM1_LDEVICE_HDLC_RECEIVER             0x0100
#define SPM1_LDEVICE_HDLC_SENDER               0x0200
#define SPM1_LDEVICE_TONE_DTMF_DETECTOR        0x0300
#define SPM1_LDEVICE_TONE_DTMF_DIALER          0x0400
#define SPM1_LDEVICE_TONE_EFFECTS_GENERATOR    0x0500
#define SPM1_LDEVICE_DATA_CONVERTER            0x0600
#define SPM1_LDEVICE_TONE_CONST_VAL_GENERATOR  0x0800
#define SPM1_LDEVICE_TONE_SINEWAVE_GENERATOR   0x0900
#define SPM1_LDEVICE_TONE_DTMF_GENERATOR       0x0a00
#define SPM1_LDEVICE_TONE_SEQUENCE_GENERATOR   0x0b00
#define SPM1_LDEVICE_TONE_SILENCE_DETECTOR     0x0c00
#define SPM1_LDEVICE_TONE_FSK_DETECTOR         0x0d00
#define SPM1_LDEVICE_DATA_MIXER                0x0e00
#define SPM1_LDEVICE_DATA_RAW_SENDER           0x0f00
#define SPM1_LDEVICE_DATA_RAW_RECEIVER         0x1000
#define SPM1_LDEVICE_DATA_CONFERENCE_BRIDGE    0x1100
#define SPM1_LDEVICE_TONE_VAL_SAMPLER          0x1200
#define SPM1_LDEVICE_TONE_MF_DETECTOR          0x1300
#define SPM1_LDEVICE_TONE_MF_DIALER            0x1400
#define SPM1_LDEVICE_TONE_DUAL_DETECTOR        0x1500
#define SPM1_LDEVICE_TONE_SINGLE_DETECTOR      0x1600
#define SPM1_LDEVICE_CST_CALLER_ID_DETECTOR    0x1700
#define SPM1_LDEVICE_CST_CALLER_ID_GENERATOR   0x1800
#define SPM1_LDEVICE_CST_COMFORT_NOISE_GENERATOR 0x1900
#define SPM1_LDEVICE_CST_VOICE_ACTIVITY_DETECTOR 0x1a00
#define SPM1_LDEVICE_CST_DATA_MODEM            0x1b00
#define SPM1_LDEVICE_CST_LI_POTS               0x1c00
#define SPM1_LDEVICE_CST_UMT_DETECTOR          0x1d00
#define SPM1_LDEVICE_CST_UMT_GENERATOR         0x1e00
#define SPM1_LDEVICE_CST_VOICE_ECHO_CANCELLER  0x1f00
#define SPM1_LDEVICE_CST_DTMF_DETECTOR         0x2000
//#define SPM1_LDEVICE_G723_DECODER              0x2100
#define SPM1_LDEVICE_G729_DECODER              0x2200
#define SPM1_LDEVICE_RPBGS                     0x2300


/*******************************************************************************
  @devnote
   DSP Notify Event codes for system device (non-logical device related)
*******************************************************************************/

#define OTXSPM1_NOTIFY_DEBUG_DATA                0x0001   // No device.
#define OTXSPM1_NOTIFY_RX_OVERFLOW               0x000e   // No device. Task loop too long. Missed Receive data. Receive overflow count (unsigned short)
#define OTXSPM1_NOTIFY_INTERNAL_ERROR            0x000f   // No device. Internal error. data is unsigned short




// hDspDevHandle for OtxSpm1 is a combination of DeviceType and BchNo, where BchNo is a connection number
// The logical device type is the 8 most significant bits of hDspDevHandle
// The BchNo is the 8 least significant bits of hDspDevHandle




/*******************************************************************************
  @devnote
   DSP IOCTL codes.
*******************************************************************************/


#define OTXSPM1_IO_RESET_BSP                             0x0101
#define OTXSPM1_IO_HDLC_SENDER_PAYLOAD_DATA              0x0102
#define OTXSPM1_IO_DTMF_DIAL                             0x0103
#define OTXSPM1_IO_GET_DEBUG_DATA                        0x0104
#define OTXSPM1_IO_INIT                                  0x0105
#define OTXSPM1_IO_SS7_SET_FILTER                        0x0106
#define OTXSPM1_IO_SS7_GET_RECEIVE_STATUS                0x0107
#define OTXSPM1_IO_VAL_SAMPLER_READ_DATA                 0x0108
//#define OTXSPM1_IO_HDLC_SENDER_GET_QUEUE_MAX_PACKET_SIZE 0x0109
#define OTXSPM1_IO_HDLC_SENDER_GET_PINGPONG_SIZE         0x0109
#define OTXSPM1_IO_SILENCE_DETECTOR_GET_STATUS           0x010A
#define OTXSPM1_IO_GET_HPI_EVENT_BUF_DIAG                0x010B
#define OTXSPM1_IO_GET_PROFILE_TIMER                     0x010C
#define OTXSPM1_IO_HDLC_SENDER_REPEAT_DATA               0x010D      // SPM  HDLC functionality
#define OTXSPM1_IO_MF_DIAL                               0x010E      // SPM  Tone functionality
#define OTXSPM1_IO_HDLC_SENDER_SS7_DATA                  0x010F      // SPM  HDLC functionality

#define OTXSPM1_IO_LI_POTS_COMMAND                       0x0110      // CST. See OtxSpm1IoInPotsLiCommandS for more details

#define OTXSPM1_IO_UMTG_DIAL                             0x0111      // CST UMTG object control
#define OTXSPM1_IO_CID_LOAD_SEQUENCE                     0x0112      // CST Called Id wrapper load sequence
#define OTXSPM1_IO_HDLC_SENDER_GENERATE_FRAME_SENT       0x0113      // OTX_ATTR_HDLC_SENDER_GENERATE_FRAME_SENT_EVENT timestamp when enabled
#define OTXSPM1_IO_HDLC_RECIEVE_MTP2_FISU_FILTER         0x0114      // MTP-2 signal FISU change indication reset
#define OTXSPM1_IO_HDLC_RECIEVE_MTP2_LSSU_FILTER         0x0115      // MTP-2 signal LSSU change indication reset
#define OTXSPM1_IO_DECODE_PAYLOAD_DATA                   0x0116


/*******************************************************************************
  @devnote
   DSP Notify Event codes.
*******************************************************************************/

#define OTXSPM1_NOTIFY_BUF_DEPLETED                         0x0080   // Hdlc Sender, Transmit buffer is depleted. No data
#define OTXSPM1_NOTIFY_DTMF_DIGIT                           0x0081   // Dtmf Detector, data: the detected digit 
#define OTXSPM1_NOTIFY_FSK_ALERT                            0x0082   // Fsk Detector, Alert tone (DTMF) is detected. No data
#define OTXSPM1_NOTIFY_FSK_COMPLETE                         0x0083   // Fsk Detector, FSK message is complete. No data
#define OTXSPM1_NOTIFY_DTMF_COMPLETE                        0x0084   // DTMF Dialler, Burst dialling completed. No data
#define OTXSPM1_NOTIFY_RX_POOL_FULL                         0x0085   // Raw Data Receiver, Data is available. Data is CH_DATA_SIZE of unsigned short
#define OTXSPM1_NOTIFY_TX_POOL_EMPTY                        0x0086   // Raw Data Sender, Need Data. No data
#define OTXSPM1_NOTIFY_HDLC_OCTETS                          0x0087   // HDLC Receiver. Octets in data
#define OTXSPM1_NOTIFY_HDLC_PROTOCOL_ERROR                  0x0088   // HDLC Receiver. Protocol error. No data
#define OTXSPM1_NOTIFY_FSK_TIMEOUT                          0x0089   // FSK Detector. No complete FSK message within timeout limit. No data
#define OTXSPM1_NOTIFY_HDLC_SENDER_BUF_DEPLETED             0x008a   // HDLC Sender. Host can send more data to us now. Not data
#define OTXSPM1_NOTIFY_HDLC_SENDER_BUF_UNDERRUN             0x008b   // HDLC Sender. Host too slow providing us with data. No data
#define OTXSPM1_NOTIFY_FSK_OCTET                            0x008c   // Fsk Detector, data: the detected octet 
#define OTXSPM1_NOTIFY_HDLC_SENDER_REPEAT_DATA              0x008d   // HDLC Sender. The Repeat Data Frame has been entered in the DSPs Repeat Data buffer
#define OTXSPM1_NOTIFY_MF_DIGIT                             0x008e   // MF Detector, High octet data: Index of coffecient for max energy, Low octet data: Index of coffecient for 2nd max energy    
#define OTXSPM1_NOTIFY_MF_COMPLETE                          0x008f   // MF Dialler, Burst dialling completed.
#define OTXSPM1_NOTIFY_MF_DETECTOR_VERBOSE_STATUS           0x0090   // MF Detector Debugging Status codes and energy levels
#define OTXSPM1_NOTIFY_MF_PAUSE                             0x0091   // MF Detector. Start of Pause detected
#define OTXSPM1_NOTIFY_DTMF_PAUSE                           0x0092   // DTMF Detector. Start of Pause detected
#define OTXSPM1_NOTIFY_SINGLE_TONE_PAUSE                    0x0096   // Single Tone Detector. Start of Pause detected
#define OTXSPM1_NOTIFY_SINGLE_TONE_DIGIT                    0x0097   // Single Tone Detector, data: Index of coffecient for max energy 
#define OTXSPM1_NOTIFY_SINGLE_DETECTOR_VERBOSE_STATUS       0x0098   // Single Tone Detector Debugging Status codes and energy levels
#define OTXSPM1_NOTIFY_DUAL_TONE_PAUSE                      0x0099   // Dual Tone Detector. Start of Pause detected
#define OTXSPM1_NOTIFY_DUAL_TONE_DIGIT                      0x009a   // Dual Tone Detector, High octet data: Index of coffecient for max energy, Low octet data: Index of coffecient for 2nd max energy
#define OTXSPM1_NOTIFY_DUAL_DETECTOR_VERBOSE_STATUS         0x009b   // Dual Tone Detector Debugging Status codes and energy levels
#define OTXSPM1_NOTIFY_FSK_VERBOSE_STATUS                   0x009c   // FSK Detector Debugging Status codes and energy levels
#define OTXSPM1_NOTIFY_HDLC_SENDER_SHORT_FRAME              0x009d   // HDLC Sender. Short frame (1 byte). Frame ignored.
#define OTXSPM1_NOTIFY_SYNC_ENABLE_TRIGGERED                0x009e   // All devices. The enabling trigger point has been reached. The device is now enabled.
#define OTXSPM1_NOTIFY_SYNC_DISABLE_TRIGGERED               0x009f   // All devices. The disabling trigger point has been reached. The device is now disabled.
#define OTXSPM1_NOTIFY_POTS_LI_RINGING_DETECTED             0x00a0   // POTS LI device. Ring signal is detected
#define OTXSPM1_NOTIFY_POTS_LI_TIP_RING_REVERSAL_DETECTED   0x00a1   // POTS LI device. Tip/Ring reversal detected. The other end hung up upon us.
//#define OTXSPM1_NOTIFY_POTS_LI_OFF_HOOK_SUCCESS             0x00a2   // POTS LI device. Off-hook is sucessful
//#define OTXSPM1_NOTIFY_POTS_LI_OFF_HOOK_FAIL                0x00a3   // POTS LI device. Off-hook failed
//#define OTXSPM1_NOTIFY_POTS_LI_ON_HOOK                      0x00a4   // POTS LI device. On-hook
//#define OTXSPM1_NOTIFY_POTS_LI_ON_HOOK_MONITOR_LINE         0x00a5   // POTS LI device. ???
//#define OTXSPM1_NOTIFY_POTS_LI_LINE_VOLTAGE_CHANGE          0x00a6   // POTS LI device. Line voltage crossed one of the thresholds. The current line voltage is passed as the data parameter
#define OTXSPM1_NOTIFY_POTS_LI_CALLER_ID_DETECTED           0x00a7   // POTS LI device. Caller ID recieved. (Not used??)
#define OTXSPM1_NOTIFY_UMTD_MESSAGE                         0x00a8   // UMT Detector. Detected new IUMTD_Message.
#define OTXSPM1_NOTIFY_UMTG_COMPLETE                        0x00a9   // UMTG Generator, Burst dialling completed. No data
#define OTXSPM1_NOTIFY_VAD_STATUS                           0x00aa   // VAD device. Currect status changed. 
#define OTXSPM1_NOTIFY_CID_DATALINK_MESSAGE                 0x00ab   // CID device. Data link message detected (CID_DataLinkMessage).
#define OTXSPM1_NOTIFY_CID_MESSAGE                          0x00ac   // CID device. Presentation message detected (CID_Message).
#define OTXSPM1_NOTIFY_CID_NUMBER                           0x00ad   // CID device.
#define OTXSPM1_NOTIFY_POTS_LI_COMMAND_COMPLETE             0x00ae   // POTS LI device. 
#define OTXSPM1_NOTIFY_POTS_LI_RINGING_END_DETECTED         0x00af   // POTS LI device. End of Ring signal is detected
#define OTXSPM1_NOTIFY_POTS_LI_AUTO_RINGING_END_DETECTED    0x00b0   // POTS LI device. Automatic report about ring end upon going off-hook
#define OTXSPM1_NOTIFY_POTS_LI_LINE_SENSE                   0x00b1   // POTS LI device. Line Sense data
#define OTXSPM1_NOTIFY_HDLC_SENDER_FRAME_SENT_COUNT_ZERO    0x00b2   // When OTX_ATTR_HDLC_SENDER_GENERATE_FRAME_SENT_EVENT timestamp is enabled, event is generated when counter is zero.
#define OTXSPM1_NOTIFY_HDLC_SENDER_FRAME_SENT               0x00b3   // OTX_ATTR_HDLC_SENDER_GENERATE_FRAME_SENT_EVENT timestamp when enabled. Count is provide by DSP and passed to application.
#define OTXSPM1_NOTIFY_SILENCE_DETECTOR_SILENCE             0x00b4   // Silence Detector: Silence
#define OTXSPM1_NOTIFY_SILENCE_DETECTOR_NON_SILENCE         0x00b5   // Silence Detector: Non-Silence


#define OTXSPM1_NOTIFY_DECODED_DATA                         0x00b8


#define OTXSPM1_NOTIFY_DPRINTF                              0x02ff   // Debug printf from logical devices within DSP.

/*******************************************************************************
  @devnote
   Structs for passing data over the IOCTL interface
*******************************************************************************/

// "In" Structs used in calls to OtxDspIoControl()
typedef struct 
{
  short          filler;     // Either 0x7E or 0xFF to be received between HDLC frames
  short          bitInvert;
  unsigned short cbMaxFrameLength;

  short          inputDataFormat;
} OtxSpm1IoInHdlcInitReceiverS;

typedef struct 
{
  short          filler;     // flag
  short          bExtractFiller; 
  short          bLsbReceivedFirst;

  short          inputDataFormat;
  short          digRxGain;
} OtxSpm1IoInRawDataInitReceiverS;

typedef struct 
{
  short          filler;     
  short          bLsbSentFirst;

  short          outputDataFormat;
  short          digTxGain;
} OtxSpm1IoInRawDataInitSenderS;


typedef struct 
{
  short          filler;     // Either 0x7E or 0xFF to be received between HDLC frames
  short          bitInvert;
  short          bSs7AutoBsn;
  unsigned short nFillers;

  short          outputDataFormat;
} OtxSpm1IoInHdlcInitSenderS;

typedef struct 
{
  short          nMinDuration;
  short          bReportPause;

  short sqCoeffFirstHarmonics[2*SPM1_DTMF_DECODER_MATRIX_SIZE];
  short sqCoeffSecondHarmonics[2*SPM1_DTMF_DECODER_MATRIX_SIZE];
  short sqKeyMapping[SPM1_DTMF_DECODER_MATRIX_SIZE*SPM1_DTMF_DECODER_MATRIX_SIZE];         

  short nThresholdPossibleTone;   // AR5(11) Threshold for possible tone
  short nThresholdDefiniteTone;   // AR5(12) Threshold for possible tone
  short nThresholdPauseEnergy;    // AR5(13) Threshold for pause energy
  short nThresholdStandardTwist;  // AR5(14) ColPeak/RowPeak if ColPeak larger than RowPeak
  short nThresholdReverseTwist;   // AR5(15) RowPeak/ColPeak if RowPeak larger than ColPeak (more likely than standard twist since lower frequency (row) are typically less attenuated then column frequencies (higher frequency)
  short nThresholdRowRelative1st; // AR5(16) First  Harmonics: RowPeak/Row2ndPlace. I.e. how many times larger is RowPeak in comparasion to the second largest Row energy.
  short nThresholdRowRelative2nd; // AR5(17) Second Harmonics: RowPeak/Row2ndPlace. I.e. how many times larger is RowPeak in comparasion to the second largest Row energy.
  short nThresholdColRelative1st; // AR5(18) First  Harmonics: ColPeak/Col2ndPlace. I.e. how many times larger is ColPeak in comparasion to the second largest Col energy.
  short nThresholdColRelative2nd; // AR5(19) First  Harmonics: ColPeak/Col2ndPlace. I.e. how many times larger is ColPeak in comparasion to the second largest Col energy.

  short          inputDataFormat;
} OtxSpm1IoInDtmfDetectorInitS;

typedef struct 
{
  short          nMinDuration;
  short          bReportPause;
  short          nVerboseStatus;

  short sqCoeffFirstHarmonics[SPM1_CO_MF_DECODER_FREQUENCIES];

  short nThresholdToneEnergy;     // Threshold for tone energy 
  short nThresholdPauseEnergy;    // Threshold for pause energy
  short nThresholdTwist;          // MaxEnergy-2ndMaxEnergy must be less than this threshold
  short nThresholdRelative3rd;    // 2ndMaxEnergy-3rdMaxEnergy must be greater than this threshold

  short          inputDataFormat;
} OtxSpm1IoInMfDetectorInitS;

typedef struct 
{
  short          nMinDuration;
  short          bReportPause;
  short          nVerboseStatus;

  short sqCoeffFirstHarmonics[SPM1_CO_DUAL_DECODER_FREQUENCIES];

  short nThresholdToneEnergy;     // Threshold for tone energy 
  short nThresholdPauseEnergy;    // Threshold for pause energy
  short nThresholdTwist;          // MaxEnergy-2ndMaxEnergy must be less than this threshold
  short nThresholdRelative3rd;    // 2ndMaxEnergy-3rdMaxEnergy must be greater than this threshold

  short          inputDataFormat;
} OtxSpm1IoInDualDetectorInitS;

typedef struct 
{
  short          nMinDuration;
  short          bReportPause;
  short          nVerboseStatus;

  short sqCoeffFirstHarmonics[SPM1_CO_SINGLE_DECODER_FREQUENCIES];

  short nThresholdVarianceEnergy; // Threshold for max energy variance from the average energy detected so far 
  short nThresholdToneEnergy;     // Threshold for tone energy 
  short nThresholdPauseEnergy;    // Threshold for pause energy

  short          inputDataFormat;
} OtxSpm1IoInSingleDetectorInitS;

typedef struct 
{
  short          toneDura;
  short          pauseDura;
  short          encStatus;  

  short          outputDataFormat;
  short          digTxGain;
} OtxSpm1IoInDtmfDiallerInitS;

typedef struct 
{
  short          nToneDuration;
  short          nPauseDuration;
  
  short sqCoeffFirstHarmonics[SPM1_CO_MF_ENCODER_FREQUENCIES];
  short sqInitialY2[SPM1_CO_MF_ENCODER_FREQUENCIES];

  short          outputDataFormat;
  short          digTxGain;
} OtxSpm1IoInMfDiallerInitS;

typedef struct 
{
  unsigned short   sqMfDigitBuffer[SPM1_MF_ENCODER_MAX_DIGITS];  // The two octets (MSB/LSB) are indexes into sqCoeffFirstHarmonics
} OtxSpm1IoInMfDialS;

typedef struct 
{
  unsigned short coBuf;  // Number of tones to be added to Play Fifo
  short int      Buf;    // First (head) element. Send other data at the end of structure in Host->Dsp buffer
                         // WARNING! - all data is WORD (2 bytes) to comply CPD tones like 0x2004
                         //    set HIGH byte with 0 in DTMF case like '0'(low),'\x00'(high)
} OtxSpm1IoInUmtgDialS;


typedef struct 
{
  short sineWaveCoeff;
  short sineWaveInitialY1;
  short sineWaveInitialY2;
  short sawtooth;
  short noiseSeed;
  short byteVariableVal;
  short toneEffectsType;

  short          digTxGain;
  short          outputDataFormat;
} OtxSpm1IoInToneEffectsGeneratorInitS;

typedef struct 
{
  short          outputDataFormat;
  short          inputDataFormat;      // for sourceChannel
} OtxSpm1IoInDataConverterInitS;

typedef struct 
{
  short          constVal;

  short          outputDataFormat;
} OtxSpm1IoInConstValGeneratorInitS;

typedef struct 
{
  short          inputDataFormat;
} OtxSpm1IoInValSamplerInitS;


typedef struct 
{
  short          nCoeff;
  short          nInitialY1;
  short          nInitialY2;
} OtxSpm1SineWaveDataS;


typedef struct 
{
  OtxSpm1SineWaveDataS sqSineWaveData[SPM1_SINEWAVE_ELEMENTS_MAX];
  short          coSineWaveData;
  short          digTxGain;
  short          outputDataFormat;
} OtxSpm1IoInSineWaveGeneratorInitS;

typedef struct 
{
  short          silenceThrHld;
  short          silenceDuration;

  short          inputDataFormat;
} OtxSpm1IoInSilenceDetectorInitS;


typedef struct 
{
  short          inputDataFormat;
} OtxSpm1IoInFskDetectorInitS;

typedef struct 
{
  short          digitBuffer[CH_TX_DIGITS_SIZE];
} OtxSpm1IoInDtmfDialS;


typedef struct 
{
  unsigned short filterMask;
} OtxSpm1IoInSs7SetFilterS;



typedef struct 
{
  unsigned short        nResult;   // Must be the first field in this struct

  unsigned short nFilterMask;
  unsigned short sqLastReceivedFisu[3];
  unsigned short nFisuCount[2];
  unsigned short nLssuCount[2];
  unsigned short nMsuCount[2];

} OtxSpm1IoOutSs7GetReceiveStatusS;

typedef struct 
{
  unsigned short        nResult;
  unsigned short cbAllocatedPingPongBuf;

} OtxSpm1IoOutHdlcSenderGetPingpongSizeS;


typedef struct 
{
  unsigned short        nResult;   // Must be the first field in this struct

  unsigned short nSample;

} OtxSpm1IoOutValSamplerReadDataS;


typedef struct 
{
  unsigned short        nResult;   // Must be the first field in this struct

  unsigned short bSilence;

} OtxSpm1IoOutSilenceDetectorGetStatusS;


typedef struct 
{
  unsigned short        nResult;   // Must be the first field in this struct

  unsigned short cwFreeBuffer;    // @field Current number of free words in the HPI Event Buffer
  unsigned short cwMinFreeBuffer; // @field Minimum number of free words in the HPI Event Buffer since the application started
  unsigned short coLostEvents;    // @field Number of lost event packets (due to a full buffer) since the application started

  unsigned short coCurrentExtQueueEvents; // @field Current number of Notify Event packets in the extended event queue (outside of HPI memory)
  unsigned short coPeakExtQueueEvents;    // @field Peak number of Notify Event packets in the extended event queue (since start)

} OtxSpm1IoOutGetHpiEventBufDiagS;

           
typedef struct 
{
  unsigned short        nResult;   // Must be the first field in this struct

  short bIsRunning;    // @field TRUE if the timer is running
  unsigned short timeDiff; // @field Time difference in 200ns ticks

} OtxSpm1IoOutGetProfileTimerS;

           
/*******************************************************************************
  @devnote
   Structs for passing data over the IOCTL interface
*******************************************************************************/

           
// Structs used for Notify Events
typedef struct 
{
  unsigned short        nData0;
  unsigned short        nData1;
  unsigned short        nData2;
  unsigned short        nData3;
  unsigned short        nData4;
  unsigned short        nData5;
  unsigned short        nData6;
  unsigned short        nData7;

  unsigned short        sqData[32];
} OtxSpm1NotifyGetDebugDataS;


/*******************************************************************************
  @devnote
   Init structures for CST based logical devices.
*******************************************************************************/

typedef struct 
{
  short nToneDurMin;  // In Samples
  short nToneDurMax;
  short nPauseDurMin;
  short nPauseDurMax;

  // WARNING: dBm values !
  // Please see SPRU638.pdf page 2-32 for more details.

                              // Defaults:
  short nThresholdSignal;     // -36.5 dBm (0x8000 * 10*log(-36.5)/1000)=0x8000*0.015
  short nThresholdPause;      // -42   dBm
  short nPauseAdapt;          // -12   dBm
  short nSignalQuality;       // -3    dBm

  short nSpuriousLevel;       // -7.4  dBc
  short nSpuriousCol;         // -1.11 dBc     "b"
  short nSpuriousRaw;         // -10.5 dBc     "a"
  short nSpuriousCol2;        // -0.72 dBc     "d"
  short nSpuriousRaw2;        // -9.3  dBc     "c"
  short nDtmfTwist;           // -11.1 dBc
  short nDtmf2Harm;           // -3.00 dBc

  // More parameters need to be added to pass all of the parameters needed for the UMTD device
  // ...


} OtxSpm1IoInUniversalMultiToneDetectorInitS;

typedef struct
{
  unsigned short  sender;   // Signal enumerator (Enumerator).
  unsigned short  type;     // Message type.
  unsigned short  duration; // Duration of last state (in frame/CH_DATA_SIZE)
} OtxCstUMTDMessage;

typedef struct {
  unsigned short nStatus;    // New status.
} OtxCstVadStatus;

/******************************************************************************\
* DAA Params Structure Type
\******************************************************************************/
typedef struct
{
    unsigned short txAttenuation;                /* analog transmit attenuation */
    unsigned short rxGain;                       /* cid receive attenuation/analog receive gain */
    unsigned short sampleRateReg7;               /* sample rate control - register 7 */
    unsigned short sampleRateReg8;               /* sample rate control - register 8 */
    unsigned short sampleRateReg9;               /* sample rate control - register 9 */
    unsigned short sampleRateReg10;              /* sample rate control - register 10 */
    unsigned short ictrl1;                       /* international control register 1 value */
    unsigned short ictrl2;                       /* international control register 2 value */
    unsigned short ictrl3;                       /* international control register 3 value */
} OtxIoInDaaParams;

enum _OtxSpm1PotsCfgSetE {
  SPM1_POTS_CFG_SET_USA       = 0,  // @emem POTS configuration for USA
  SPM1_POTS_CFG_SET_GERMANY   = 1,  // @emem POTS configuration for Germany
  SPM1_POTS_CFG_SET_UK        = 2,  // @emem POTS configuration for United Kingdom
  SPM1_POTS_CFG_SET_FRANCE    = 3,  // @emem POTS configuration for France
  SPM1_POTS_CFG_SET_AUSTRALIA = 4,  // @emem POTS configuration for Australia
  SPM1_POTS_CFG_SET_JAPAN     = 5   // @emem POTS configuration for Japan
};

typedef enum _OtxSpm1PotsCfgSetE OtxSpm1PotsCfgSetE;

typedef struct {
  unsigned short       nLineVoltageMonitorPollingInterval;  // in milliseconds

  // Line voltage thresholds. In increasing voltage levels.
  unsigned short       nMultipleOffHookThreshold;  // 0                         < LineVoltage <= nMultipleOffHookThreshold :  No line is connected
  unsigned short       nSingleOffHookThreshold;    // nMultipleOffHookThreshold < LineVoltage <= nSingleOffHookThreshold   :  Multiple phones are off-hook
  unsigned short       nOnHookThreshold;           // nSingleOffHookThreshold   < LineVoltage <= nOnOffHookThreshold       :  Single phone is off-hook
                                                   // nOnHookThreshold          < LineVoltage                              :  On-hook

  short                bUseGroundStart;  // if set then the PotsLi device will ignore any ringing events
                                         // These are instead handled by the OTX_DEVICE_GROUND_START physical device

} OtxPotsCommonParams;

typedef struct 
{
  unsigned short size;         /* Size of structure. */
  unsigned short nDataFormats; /* Stream data format: Low byte = RX Data format, High byte = TX Data format */


  OtxIoInDaaParams     DaaParams;
  OtxPotsCommonParams  CommonPar;
  //OtxSpm1PotsCfgSetE   eCfgSet;

} OtxSpm1IoInPotsLiInitS;


/*******************************************************************************
  @devnote
   Command structures for CST based logical devices.
*******************************************************************************/

           
/*******************************************************************************
  @devnote
   Command structure for Pots Li logical device.
  
  @note
    Available commands: 
      pdc_PULSE_GEN, nDigit
      pdc_ON_HOOK, 0
      pdc_OFF_HOOK, 0


*******************************************************************************/
typedef struct {
  unsigned short nCommand;  // CST command.
  unsigned short nParam1;
  unsigned short nParam2;

} OtxSpm1IoInPotsLiCommandS;


typedef struct {
  unsigned short nResult;          // Must be the first field in this struct

  unsigned short nDaaResult[2];    // Raw result code from DAA Driver

} OtxSpm1IoOutPotsLiCommandS;

typedef struct {
  unsigned short nSeqID;           // Sequence ID
  unsigned short nCommand;        

  unsigned short nDaaResult[2];    // Raw result code from DAA Driver
 

} OtxSpm1IoOutPotsLiCommandCompleteS;
           
typedef struct 
{
  short          nDurationMs;  // duration in milliseconds

} OtxSpm1IoInPotsIoInHookFlashS;


typedef struct OtxSpm1IoInPotsIoInPulseDialS
{
  short          nDisconnectMs;
  short          nConnectMs;
  short          nInterPulseGroupMs;
  short          sqPulseGroups[SPM1_POTS_PULSE_GROUPS_MAX];
  short          coPulseGroups;

} OtxSpm1IoInPotsIoInPulseDialS;


typedef struct{
  unsigned short nCode;         // see enum Spm1CidErrorE
  unsigned short nDigits;
} OtxSpm1IoOutCidMessageHead;

// ---------------------------- MIXER ---------------------------------------

#define MIXER_MAX_TS 32

typedef struct {
  unsigned short nOutDataFormats; /* Stream data format: Low byte = RX Data format, High byte = TX Data format */
  unsigned short size;         /* Size of structure. */
  unsigned short coInputsMax;
  unsigned short nInDataFormats[MIXER_MAX_TS]; /* Stream data format: Low byte = RX Data format, High byte = TX Data format */
  short nInDigGains[MIXER_MAX_TS]; 
  short nOutDigGain;

  short          reserved;
} CST_PROFILE_MIXER;

typedef enum OtxSpm1DataProcessModeTagE{
  DIRECT_EVENT_MODE  = 0x00,
  BCH_MODE           = 0x01,
  DIRECT_INCALL_MODE = 0x02
} OtxSpm1DataProcessModeE;


typedef struct {
  /*OtxSpm1DataProcessModeE*/short eProcessMode;
  short nOutFormat;
} OtxSpm1G729InitS;


#if defined(_TMS320C5XX)
// No pack pragma needed for the DSP compiler
#elif defined(__TMS320C55X__)
// No pack pragma needed for the DSP compiler
#elif defined(xdc_target_name__) && xdc_target_name__==C64P
// No pack pragma needed for the DSP compiler
#elif defined (__linux__)
// GNU C has no pragma pack pop support
#else
#pragma pack()
#endif


/*--------------------------------------------------------------------------*\

  Copyright (C) Odin TeleSystems Inc., 1999-2009


  All rights reserved. No part of this computer program may be reproduced,
  stored in any retrieval system, or transmitted, in any form or by any
  means, electronic, mechanical, photocopying, recording, or otherwise,
  without the prior written permission of Odin TeleSystems Inc., or without
  a permission stipulated in the agreement/contract under which this computer
  program was supplied.

\*--------------------------------------------------------------------------*/
#endif /* DSP_OTXSPM_IO_H */
